@@ -15,21 +15,21 @@
 
 pub(crate) use {
   crate::{
-    alias::Alias, analyzer::Analyzer, assignment::Assignment,
+    alias::Alias, analyzer::Analyzer, argument_parser::ArgumentParser, assignment::Assignment,
     assignment_resolver::AssignmentResolver, ast::Ast, attribute::Attribute, binding::Binding,
     color::Color, color_display::ColorDisplay, command_ext::CommandExt, compilation::Compilation,
     compile_error::CompileError, compile_error_kind::CompileErrorKind, compiler::Compiler,
     condition::Condition, conditional_operator::ConditionalOperator, config::Config,
     config_error::ConfigError, constants::constants, count::Count, delimiter::Delimiter,
     dependency::Dependency, dump_format::DumpFormat, enclosure::Enclosure, error::Error,
-    evaluator::Evaluator, expression::Expression, fragment::Fragment, function::Function,
-    interrupt_guard::InterruptGuard, interrupt_handler::InterruptHandler, item::Item,
-    justfile::Justfile, keyed::Keyed, keyword::Keyword, lexer::Lexer, line::Line, list::List,
-    load_dotenv::load_dotenv, loader::Loader, module_path::ModulePath, name::Name,
-    namepath::Namepath, ordinal::Ordinal, output::output, output_error::OutputError,
-    parameter::Parameter, parameter_kind::ParameterKind, parser::Parser, platform::Platform,
-    platform_interface::PlatformInterface, position::Position, positional::Positional, ran::Ran,
-    range_ext::RangeExt, recipe::Recipe, recipe_context::RecipeContext,
+    evaluator::Evaluator, execution_context::ExecutionContext, expression::Expression,
+    fragment::Fragment, function::Function, interrupt_guard::InterruptGuard,
+    interrupt_handler::InterruptHandler, item::Item, justfile::Justfile, keyed::Keyed,
+    keyword::Keyword, lexer::Lexer, line::Line, list::List, load_dotenv::load_dotenv,
+    loader::Loader, module_path::ModulePath, name::Name, namepath::Namepath, ordinal::Ordinal,
+    output::output, output_error::OutputError, parameter::Parameter, parameter_kind::ParameterKind,
+    parser::Parser, platform::Platform, platform_interface::PlatformInterface, position::Position,
+    positional::Positional, ran::Ran, range_ext::RangeExt, recipe::Recipe,
     recipe_resolver::RecipeResolver, recipe_signature::RecipeSignature, scope::Scope,
     search::Search, search_config::SearchConfig, search_error::SearchError, set::Set,
     setting::Setting, settings::Settings, shebang::Shebang, shell::Shell,
@@ -39,15 +39,27 @@ pub(crate) use {
     unresolved_recipe::UnresolvedRecipe, use_color::UseColor, variables::Variables,
     verbosity::Verbosity, warning::Warning,
   },
+  camino::Utf8Path,
+  derivative::Derivative,
+  edit_distance::edit_distance,
+  lexiclean::Lexiclean,
+  libc::EXIT_FAILURE,
+  log::{info, warn},
+  regex::Regex,
+  serde::{
+    ser::{SerializeMap, SerializeSeq},
+    Serialize, Serializer,
+  },
+  snafu::{ResultExt, Snafu},
   std::{
     borrow::Cow,
     cmp,
-    collections::{BTreeMap, BTreeSet, HashMap},
+    collections::{BTreeMap, BTreeSet, HashMap, HashSet},
     env,
     ffi::OsString,
     fmt::{self, Debug, Display, Formatter},
     fs,
-    io::{self, Write},
+    io::{self, Read, Seek, Write},
     iter::{self, FromIterator},
     mem,
     ops::Deref,
@@ -59,23 +71,10 @@ pub(crate) use {
     sync::{Mutex, MutexGuard, OnceLock},
     vec,
   },
-  {
-    camino::Utf8Path,
-    derivative::Derivative,
-    edit_distance::edit_distance,
-    lexiclean::Lexiclean,
-    libc::EXIT_FAILURE,
-    log::{info, warn},
-    regex::Regex,
-    serde::{
-      ser::{SerializeMap, SerializeSeq},
-      Serialize, Serializer,
-    },
-    snafu::{ResultExt, Snafu},
-    strum::{Display, EnumDiscriminants, EnumString, IntoStaticStr},
-    typed_arena::Arena,
-    unicode_width::{UnicodeWidthChar, UnicodeWidthStr},
-  },
+  strum::{Display, EnumDiscriminants, EnumString, IntoStaticStr},
+  tempfile::tempfile,
+  typed_arena::Arena,
+  unicode_width::{UnicodeWidthChar, UnicodeWidthStr},
 };
 
 #[cfg(test)]
@@ -114,6 +113,7 @@ pub mod summary;
 
 mod alias;
 mod analyzer;
+mod argument_parser;
 mod assignment;
 mod assignment_resolver;
 mod ast;
@@ -139,6 +139,7 @@ mod dump_format;
 mod enclosure;
 mod error;
 mod evaluator;
+mod execution_context;
 mod expression;
 mod fragment;
 mod function;
@@ -169,7 +170,6 @@ mod positional;
 mod ran;
 mod range_ext;
 mod recipe;
-mod recipe_context;
 mod recipe_resolver;
 mod recipe_signature;
 mod run;
