@@ -4,6 +4,7 @@ use super::*;
 pub(crate) struct Source<'src> {
   pub(crate) file_depth: u32,
   pub(crate) file_path: Vec<PathBuf>,
+  pub(crate) import_offsets: Vec<usize>,
   pub(crate) namepath: Namepath<'src>,
   pub(crate) path: PathBuf,
   pub(crate) submodule_depth: u32,
@@ -15,6 +16,7 @@ impl<'src> Source<'src> {
     Self {
       file_depth: 0,
       file_path: vec![path.into()],
+      import_offsets: Vec::new(),
       namepath: Namepath::default(),
       path: path.into(),
       submodule_depth: 0,
@@ -22,7 +24,7 @@ impl<'src> Source<'src> {
     }
   }
 
-  pub(crate) fn import(&self, path: PathBuf) -> Self {
+  pub(crate) fn import(&self, path: PathBuf, import_offset: usize) -> Self {
     Self {
       file_depth: self.file_depth + 1,
       file_path: self
@@ -31,6 +33,12 @@ impl<'src> Source<'src> {
         .into_iter()
         .chain(iter::once(path.clone()))
         .collect(),
+      import_offsets: self
+        .import_offsets
+        .iter()
+        .copied()
+        .chain(iter::once(import_offset))
+        .collect(),
       namepath: self.namepath.clone(),
       path,
       submodule_depth: self.submodule_depth,
@@ -47,6 +55,7 @@ impl<'src> Source<'src> {
         .into_iter()
         .chain(iter::once(path.clone()))
         .collect(),
+      import_offsets: Vec::new(),
       namepath: self.namepath.join(name),
       path: path.clone(),
       submodule_depth: self.submodule_depth + 1,
