@@ -1,9 +1,4 @@
-use {
-  super::*,
-  clap_mangen::Man,
-  std::io::{Read, Seek},
-  tempfile::tempfile,
-};
+use {super::*, clap_mangen::Man};
 
 const INIT_JUSTFILE: &str = "default:\n    echo 'Hello, world!'\n";
 
@@ -20,7 +15,7 @@ pub(crate) enum Subcommand {
     overrides: BTreeMap<String, String>,
   },
   Completions {
-    shell: clap_complete::Shell,
+    shell: completions::Shell,
   },
   Dump,
   Edit,
@@ -155,7 +150,7 @@ impl Subcommand {
         };
 
         match Self::run_inner(config, loader, arguments, overrides, &search) {
-          Err((err @ Error::UnknownRecipes { .. }, true)) => {
+          Err((err @ Error::UnknownRecipe { .. }, true)) => {
             match search.justfile.parent().unwrap().parent() {
               Some(parent) => {
                 unknown_recipes_errors.get_or_insert(err);
@@ -236,7 +231,15 @@ impl Subcommand {
       return Err(Error::NoChoosableRecipes);
     }
 
-    let chooser = chooser.map_or_else(|| config::chooser_default(&search.justfile), From::from);
+    let chooser = if let Some(chooser) = chooser {
+      OsString::from(chooser)
+    } else {
+      let mut chooser = OsString::new();
+      chooser.push("fzf --multi --preview 'just --unstable --color always --justfile \"");
+      chooser.push(&search.justfile);
+      chooser.push("\" --show {}'");
+      chooser
+    };
 
     let result = justfile
       .settings
@@ -296,68 +299,8 @@ impl Subcommand {
     justfile.run(config, search, overrides, &recipes)
   }
 
-  fn completions(shell: clap_complete::Shell) -> RunResult<'static, ()> {
-    use clap_complete::Shell;
-
-    fn replace(haystack: &mut String, needle: &str, replacement: &str) -> RunResult<'static, ()> {
-      if let Some(index) = haystack.find(needle) {
-        haystack.replace_range(index..index + needle.len(), replacement);
-        Ok(())
-      } else {
-        Err(Error::internal(format!(
-          "Failed to find text:\n{needle}\n…in completion script:\n{haystack}"
-        )))
-      }
-    }
-
-    let mut script = {
-      let mut tempfile = tempfile().map_err(|io_error| Error::TempfileIo { io_error })?;
-
-      clap_complete::generate(
-        shell,
-        &mut crate::config::Config::app(),
-        env!("CARGO_PKG_NAME"),
-        &mut tempfile,
-      );
-
-      tempfile
-        .rewind()
-        .map_err(|io_error| Error::TempfileIo { io_error })?;
-
-      let mut buffer = String::new();
-
-      tempfile
-        .read_to_string(&mut buffer)
-        .map_err(|io_error| Error::TempfileIo { io_error })?;
-
-      buffer
-    };
-
-    match shell {
-      Shell::Bash => {
-        for (needle, replacement) in completions::BASH_COMPLETION_REPLACEMENTS {
-          replace(&mut script, needle, replacement)?;
-        }
-      }
-      Shell::Fish => {
-        script.insert_str(0, completions::FISH_RECIPE_COMPLETIONS);
-      }
-      Shell::PowerShell => {
-        for (needle, replacement) in completions::POWERSHELL_COMPLETION_REPLACEMENTS {
-          replace(&mut script, needle, replacement)?;
-        }
-      }
-
-      Shell::Zsh => {
-        for (needle, replacement) in completions::ZSH_COMPLETION_REPLACEMENTS {
-          replace(&mut script, needle, replacement)?;
-        }
-      }
-      _ => {}
-    }
-
-    println!("{}", script.trim());
-
+  fn completions(shell: completions::Shell) -> RunResult<'static, ()> {
+    println!("{}", shell.script()?);
     Ok(())
   }
 
@@ -485,7 +428,9 @@ impl Subcommand {
       module = module
         .modules
         .get(name)
-        .ok_or_else(|| Error::UnknownSubmodule { path: path.clone() })?;
+        .ok_or_else(|| Error::UnknownSubmodule {
+          path: path.to_string(),
+        })?;
     }
 
     Self::list_module(config, module, 0);
@@ -645,7 +590,9 @@ impl Subcommand {
       module = module
         .modules
         .get(name)
-        .ok_or_else(|| Error::UnknownSubmodule { path: path.clone() })?;
+        .ok_or_else(|| Error::UnknownSubmodule {
+          path: path.to_string(),
+        })?;
     }
 
     let name = path.path.last().unwrap();
@@ -659,8 +606,8 @@ impl Subcommand {
       println!("{}", recipe.color_display(config.color.stdout()));
       Ok(())
     } else {
-      Err(Error::UnknownRecipes {
-        recipes: vec![name.to_owned()],
+      Err(Error::UnknownRecipe {
+        recipe: name.to_owned(),
         suggestion: module.suggest_recipe(name),
       })
     }
